#|
#|  Copyright The Telepact Authors
#|
#|  Licensed under the Apache License, Version 2.0 (the "License");
#|  you may not use this file except in compliance with the License.
#|  You may obtain a copy of the License at
#|
#|  https://www.apache.org/licenses/LICENSE-2.0
#|
#|  Unless required by applicable law or agreed to in writing, software
#|  distributed under the License is distributed on an "AS IS" BASIS,
#|  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#|  See the License for the specific language governing permissions and
#|  limitations under the License.
#|

class SerializationError(Exception):
    def __init__(
        self,
        message: str = "telepact serialization failed",
        *,
        context: str | None = None,
        cause: BaseException | None = None,
    ) -> None:
        if context:
            message = f"{message} while {context}"
        if cause is not None:
            message = f"{message}: {cause}"
        super().__init__(message)
        self.context = context
        self.cause = cause
