#|
#|  Copyright The Telepact Authors
#|
#|  Licensed under the Apache License, Version 2.0 (the "License");
#|  you may not use this file except in compliance with the License.
#|  You may obtain a copy of the License at
#|
#|  https://www.apache.org/licenses/LICENSE-2.0
#|
#|  Unless required by applicable law or agreed to in writing, software
#|  distributed under the License is distributed on an "AS IS" BASIS,
#|  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#|  See the License for the specific language governing permissions and
#|  limitations under the License.
#|

from typing import Any, TYPE_CHECKING

from ...internal.binary.BinaryEncodingCache import BinaryEncodingCache

if TYPE_CHECKING:
    from .BinaryEncoding import BinaryEncoding

class DefaultBinaryEncodingCache(BinaryEncodingCache):

    def __init__(self) -> None:
        self.recent_binary_encoders: dict[int, 'BinaryEncoding'] = {}

    def add(self, checksum: int, binary_encoding_map: dict[str, int]) -> None:
        from .BinaryEncoding import BinaryEncoding        
        binary_encoding = BinaryEncoding(binary_encoding_map, checksum)
        self.recent_binary_encoders[checksum] = binary_encoding

    def get(self, checksum: int) -> 'BinaryEncoding':
        return self.recent_binary_encoders[checksum]

    def remove(self, checksum: int) -> None:
        self.recent_binary_encoders.pop(checksum, None)